﻿// (C) Copyright 2002-2009 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//
using System;
using Autodesk.AutoCAD.Runtime;
using Autodesk.AutoCAD.ApplicationServices;
using Autodesk.AutoCAD.DatabaseServices;
using Autodesk.AutoCAD.Geometry;
using Autodesk.AutoCAD.EditorInput;
using Autodesk.AutoCAD.Colors;
using System.Collections.Generic;

// This line is not mandatory, but improves loading performances
[assembly: CommandClass(typeof(Autodesk.AutoCAD.ThreeDimensions.MyCommands))]

namespace Autodesk.AutoCAD.ThreeDimensions
{
  // This class is instantiated by AutoCAD for each document when
  // a command is called by the user the first time in the context
  // of a given document. In other words, non static data in this class
  // is implicitly per-document!
  public partial class MyCommands
  {
    // The CommandMethod attribute can be applied to any public  member 
    // function of any public class.
    // The function should take no arguments and return nothing.
    // If the method is an intance member then the enclosing class is 
    // intantiated for each document. If the member is a static member then
    // the enclosing class is NOT intantiated.
    //
    // NOTE: CommandMethod has overloads where you can provide helpid and
    // context menu.

    // create a lamp with 2 guides mirrored n each side
    [CommandMethod("planeSurface")]
    public void planeSurface()
    {
      // get the normal vars
      Document doc = Application.DocumentManager.MdiActiveDocument;
      Database db = doc.Database;
      Editor ed = doc.Editor;

      // create a new polyline in memory to desribe the outline
      Polyline polyline = new Polyline();
      polyline.SetDatabaseDefaults();

      int vertex = 0;
      // set up the input
      PromptPointOptions opts = new PromptPointOptions("Pick Start Point");
      PromptPointResult res = ed.GetPoint(opts);
      // if ok
      if (res.Status == PromptStatus.OK)
      {
        // add the point
        polyline.AddVertexAt(vertex++, new Point2d(res.Value.X, res.Value.Y), 0, 0, 0);
        opts.UseBasePoint = true;
        opts.AllowNone = true;
        opts.Message = "Pick Next Point or Enter to <Finish>";
        // loop while not Enter
        do 
        {
          opts.BasePoint = res.Value;
          // get the input
          res = ed.GetPoint(opts);
          // if canceling
          if (res.Status == PromptStatus.Cancel)
            return;
          // if quiting the loop
          if (res.Status == PromptStatus.None)
            break;

          // add the point
          polyline.AddVertexAt(vertex++, new Point2d(res.Value.X, res.Value.Y), 0, 0, 0);

        }while(res.Status == PromptStatus.OK);
      
        // set to closed
        polyline.Closed = true;

        // create a collection of 
        DBObjectCollection entities = new DBObjectCollection();
        entities.Add(polyline);

        using (BlockTableRecord curSpace = db.CurrentSpaceId.Open(OpenMode.ForWrite) as BlockTableRecord)
        {
          // create regions from the list of entities
          DBObjectCollection regions = Region.CreateFromCurves(entities);
          // now loop the regions
          foreach (Region region in regions)
          {
            using (PlaneSurface planeSurface = new PlaneSurface())
            {
              planeSurface.CreateFromRegion(region);
              curSpace.AppendEntity(planeSurface);
            }
          }
        }
      }
    }
  }
}