﻿// (C) Copyright 2002-2009 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//
using System;
using Autodesk.AutoCAD.Runtime;
using Autodesk.AutoCAD.ApplicationServices;
using Autodesk.AutoCAD.DatabaseServices;
using Autodesk.AutoCAD.Geometry;
using Autodesk.AutoCAD.EditorInput;

namespace Autodesk.AutoCAD.ThreeDimensions
{
  public class Utils
  {
    static public double pi = System.Math.Acos(-1);

    static public Polyline CreateStarOutline(Point2d pnt, double rotationAng)
    {
      Polyline poly = new Polyline();
      poly.SetDatabaseDefaults();
      int vertex = 0;
      poly.AddVertexAt(vertex++, pnt, 0, 0, 0);
      poly.AddVertexAt(vertex++, pnt + new Vector2d(10, 0), 0, 0, 0);
      poly.AddVertexAt(vertex++, pnt + new Vector2d(10, 10), 0, 0, 0);
      poly.AddVertexAt(vertex++, pnt + new Vector2d(0, 10), 0, 0, 0);
      poly.AddVertexAt(vertex++, pnt + new Vector2d(0, 20), 0, 0, 0);
      poly.AddVertexAt(vertex++, pnt + new Vector2d(-10, 20), 0, 0, 0);
      poly.AddVertexAt(vertex++, pnt + new Vector2d(-10, 20), 0, 0, 0);
      poly.AddVertexAt(vertex++, pnt + new Vector2d(-10, 10), 0, 0, 0);
      poly.AddVertexAt(vertex++, pnt + new Vector2d(-20, 10), 0, 0, 0);
      poly.AddVertexAt(vertex++, pnt + new Vector2d(-20, 0), 0, 0, 0);
      poly.AddVertexAt(vertex++, pnt + new Vector2d(-10, 0), 0, 0, 0);
      poly.AddVertexAt(vertex++, pnt + new Vector2d(-10, -10), 0, 0, 0);
      poly.AddVertexAt(vertex++, pnt + new Vector2d(0, -10), 0, 0, 0);
      poly.Closed = true;

      Matrix3d rotation = Matrix3d.Rotation(rotationAng / 180 * pi, Vector3d.ZAxis, new Point3d(pnt.X, pnt.Y, 0));
      poly.TransformBy(rotation);

      return poly;
    }

    static public Circle CreateCircle(Point3d pnt, double radius)
    {
      Circle circle = new Circle(pnt, Vector3d.ZAxis, radius);
      circle.SetDatabaseDefaults();

      return circle;
    }
  }

  // experimental class for extracting the Dynamic properties from a Solid
  /*public class OPMPerInstancePropertyExtensionFactory : RXObject
  {
    public extern virtual RXClass isA();
    public static RXClass gpDesc;
    public extern static RXClass desc();
    public static OPMPerInstancePropertyExtensionFactory cast(RXObject inPtr) 
    {
      if (inPtr == null || (inPtr as OPMPerInstancePropertyExtensionFactory) == null)
        return null;

      return inPtr as OPMPerInstancePropertyExtensionFactory; 
    }
    public extern static void rxInit();
    public extern static void rxInit(System.IntPtr ptr);
    public extern virtual void destructor();
    // Retrieves the OPMPerInstancePropertyExtension instance per class. If the 
    // extension has not been added it creates it. Note that the factory manages
    // the lifetime of the extension object, so callers should not delete the memory
    // referenced by the returned pointer.
    public extern virtual OPMPerInstancePropertyExtension CreateOPMPerInstancePropertyExtension(RXClass pClass);
  };

  public class OPMPerInstancePropertyExtension : RXObject
  {
    public virtual RXClass isA();
    public static RXClass gpDesc;
    public static RXClass desc(){}
    public static OPMPerInstancePropertyExtension cast(RXObject inPtr) 
    {
      if (inPtr == null || (inPtr as OPMPerInstancePropertyExtension) == null)
        return null;

      return inPtr as OPMPerInstancePropertyExtension; 
    }
    public static void rxInit(){}
    public static void rxInit(System.IntPtr ptr){}
    public virtual void destructor(){}
    // Adds a property source name 
    public virtual bool AddObjectPropertySourceName(System.IntPtr pName){}
    // Removes a property source name
    public virtual bool RemoveObjectPropertySourceName(System.IntPtr pName){}
    // Retrieves the property source names stored on the extension object. 
    // Returned variant may be VT_EMPTY if no sources are defined.
    public virtual bool GetObjectPropertySourceNames(object pNames){}
  };*/

}
